@@ -76,7 +76,7 @@ struct h2con {
     uint32_t rused;
 
     uint32_t h2_cid;
-    uint32_t h2_sid;
+    uint32_t h2_sid;  /* unused; server push (not implemented) */
      int32_t sent_goaway;
     unix_time64_t sent_settings;
     uint32_t s_header_table_size;      /* SETTINGS_HEADER_TABLE_SIZE      */
